"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rpc_1 = require("./rpc");
var MuRemoteRPCClient = (function () {
    function MuRemoteRPCClient(client, rpc) {
        this.sessionId = client.sessionId;
        this.rpc = rpc;
    }
    return MuRemoteRPCClient;
}());
exports.MuRemoteRPCClient = MuRemoteRPCClient;
function findClient(clients, id) {
    for (var i = 0; i < clients.length; ++i) {
        if (clients[i].sessionId === id) {
            return i;
        }
    }
    return -1;
}
function removeItem(array, index) {
    array[index] = array[array.length - 1];
    array.pop();
}
var uniqueNumber = (function () {
    var next = 0;
    return function () { return next++; };
})();
var MuRPCServer = (function () {
    function MuRPCServer(server, schema) {
        this.clients = [];
        this._callbacks = {};
        this.server = server;
        this.schema = schema;
        var schemaUnfolded = rpc_1.unfoldRPCProtocolSchema(schema);
        this._callProtocol = server.protocol(schemaUnfolded[0]);
        this._responseProtocol = server.protocol(schemaUnfolded[1]);
        this._errorProtocol = server.protocol(rpc_1.MuRPCErrorProtocolSchema);
    }
    MuRPCServer.prototype._createRPCToClient = function (clientId) {
        var _this = this;
        var rpc = {};
        Object.keys(this.schema.client).forEach(function (method) {
            rpc[method] = function (arg, next) {
                var id = uniqueNumber();
                _this._callbacks[clientId][id] = next;
                _this._callProtocol.clients[clientId].message[method]({
                    base: _this.schema.client[method][0].clone(arg),
                    id: id,
                });
            };
        });
        return rpc;
    };
    MuRPCServer.prototype.configure = function (spec) {
        var _this = this;
        this._callProtocol.configure({
            message: (function () {
                var handlers = {};
                Object.keys(_this.schema.server).forEach(function (method) {
                    handlers[method] = function (client_, _a) {
                        var base = _a.base, id = _a.id;
                        var clientIdx = findClient(_this.clients, client_.sessionId);
                        var client = _this.clients[clientIdx];
                        spec.rpc[method](base, function (err, response) {
                            if (err) {
                                _this._errorProtocol.clients[client.sessionId].message.error({ message: err, id: id });
                            }
                            else {
                                _this._responseProtocol.clients[client.sessionId].message[method]({
                                    base: _this.schema.server[method][1].clone(response),
                                    id: id,
                                });
                            }
                        }, client);
                    };
                });
                return handlers;
            })(),
            ready: function () {
                if (spec && spec.ready) {
                    spec.ready();
                }
            },
            connect: function (client_) {
                var client = new MuRemoteRPCClient(client_, _this._createRPCToClient(client_.sessionId));
                _this.clients.push(client);
                _this._callbacks[client_.sessionId] = {};
                if (spec && spec.connect) {
                    spec.connect(client);
                }
            },
            disconnect: function (client) {
                var clientIdx = findClient(_this.clients, client.sessionId);
                if (spec && spec.disconnect) {
                    spec.disconnect(_this.clients[clientIdx]);
                }
                removeItem(_this.clients, clientIdx);
                delete _this._callbacks[client.sessionId];
            },
            close: function () {
                if (spec && spec.close) {
                    spec.close();
                }
            },
        });
        this._responseProtocol.configure({
            message: (function () {
                var handlers = {};
                Object.keys(_this.schema.client).forEach(function (method) {
                    handlers[method] = function (client, _a) {
                        var base = _a.base, id = _a.id;
                        var clientId = client.sessionId;
                        if (_this._callbacks[clientId] && _this._callbacks[clientId][id]) {
                            _this._callbacks[clientId][id](_this.schema.client[method][1].clone(base));
                            delete _this._callbacks[clientId][id];
                        }
                    };
                });
                return handlers;
            })(),
        });
        this._errorProtocol.configure({
            message: {
                error: function (client, _a) {
                    var message = _a.message, id = _a.id;
                    var clientId = client.sessionId;
                    if (_this._callbacks[clientId]) {
                        delete _this._callbacks[clientId][id];
                    }
                },
            },
        });
    };
    return MuRPCServer;
}());
exports.MuRPCServer = MuRPCServer;
//# sourceMappingURL=server.js.map